mod common;
use crate::common::pod_request::{deploy_plugin, migrate_plugin_definition, start_plugin};
use common::test_data::TestData;
use libpod::api_model::{PluginStatus, PluginStatusRes};
use serde_json::json;
use test_context::test_context;

#[test_context(TestData)]
#[tokio::test]
async fn test_plugin_state(ctx: &mut TestData) {
    migrate_plugin_definition(&ctx.pod_client).await;

    // Request status of all plugins on fresh instance, should be none
    let payload = json!(
        {
            "plugins" : []
        }
    );

    let response = ctx.pod_client.post_to(payload, "plugin/status").await;

    assert!(response.status().is_success());
    assert_eq!(
        response
            .json::<PluginStatusRes>()
            .await
            .unwrap()
            .plugins
            .len(),
        0
    );

    deploy_plugin(&ctx.pod_client).await;

    start_plugin(ctx).await;

    // Request status of all plugins
    let payload = json!(
        {
            "plugins" : []
        }
    );

    let response = ctx.pod_client.post_to(payload, "plugin/status").await;

    assert!(response.status().is_success());

    let response = response.json::<PluginStatusRes>().await.unwrap();
    assert_eq!(response.plugins.len(), 1);

    let plugin_run_id = "4fa2094c-51d6-4874-a135-9017fcdedf16";

    assert!(matches!(
        response.plugins.get(plugin_run_id).unwrap(),
        PluginStatus::Running(_)
    ));

    // Request status of selected plugins
    let payload = json!(
        {
            "plugins" : [plugin_run_id, "invalid-id"]
        }
    );

    let response = ctx.pod_client.post_to(payload, "plugin/status").await;

    assert!(response.status().is_success());

    let response = response.json::<PluginStatusRes>().await.unwrap();
    assert_eq!(response.plugins.len(), 1);
    assert!(matches!(
        response.plugins.get(plugin_run_id).unwrap(),
        PluginStatus::Running(_)
    ));

    // Request status of selected invalid plugin
    let payload = json!(
        {
            "plugins" : ["invalid-id"]
        }
    );

    let response = ctx.pod_client.post_to(payload, "plugin/status").await;

    assert!(response.status().is_success());

    let response = response.json::<PluginStatusRes>().await.unwrap();
    assert_eq!(response.plugins.len(), 0);

    ctx.stop_containers().await;

    // Request status of selected plugin, that does not run
    let payload = json!(
        {
            "plugins" : [plugin_run_id, "invalid-id"]
        }
    );

    let response = ctx.pod_client.post_to(payload, "plugin/status").await;

    assert!(response.status().is_success());

    let response = response.json::<PluginStatusRes>().await.unwrap();
    assert_eq!(response.plugins.len(), 1);
    assert!(matches!(
        response.plugins.get(plugin_run_id).unwrap(),
        PluginStatus::Unreachable
    ));
}
